==> blcr-0.5.0.tar.gz: blcr-0.5.0/examples/io_bench/io_bench.c <==
/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2006, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: io_bench.c,v 1.1 2006/06/14 22:16:43 phargrov Exp $
 */

#define _LARGEFILE64_SOURCE 1	/* For O_LARGEFILE */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdint.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
 
#include "libcr.h"

extern uint64_t gettimeofday_us(void) {
  uint64_t retval;
  struct timeval tv;
  gettimeofday(&tv, NULL);
  retval = ((uint64_t)tv.tv_sec) * 1000000 + (uint64_t)tv.tv_usec;
  return retval;
}

int work(int argc, const char **argv) {
    size_t remain;
    int mb;

    if (argc != 2) {
      printf("ERROR: %s requires 1 argument, the memory size in megabytes\n", argv[0]);
      exit(1);
    }
    mb = atoi(argv[1]);
    if (mb <= 0) {
      printf("ERROR: '%s' is not a positive integer\n", argv[1]);
      exit(1);
    }

    remain = mb * 1024 * 1024;
    while (remain) {
      size_t try;
      for (try = remain; try != 0; try /= 2) {
        char *ptr = malloc(try);
	if (ptr != NULL) {
	  int i;
	  /* Touch each page (more than once if page size > 4k */
	  for (i = 0; i < try; i += 4096) {
	    ptr[i] = (char)1;
	  }
	  remain -= try;
	  break;
	}
      }
      if (try == 0) {
        printf("ERROR: failed to allocate %d megabytes of memory\n", mb);
        exit(1);
      }
    }

    return mb;
}

int main(int argc, const char **argv)
{
    char filename[40];
    cr_client_id_t my_id;
    uint64_t start_time, end_time;
    int rc, fd, mb;

    setlinebuf(stdout);

    snprintf(filename, sizeof(filename), "context.%d", (int)getpid());
    fd = open(filename, O_WRONLY|O_CREAT|O_NOCTTY|O_NONBLOCK|O_TRUNC|O_LARGEFILE, 0400);
    if (fd < 0) {
	printf("ERROR: open(%s) failed: %s\n", filename, strerror(errno));
	exit(-1);
    }

    my_id = cr_init();
    if (my_id < 0) {
	printf("ERROR: cr_init() failed, returning %d\n", my_id);
	exit(-1);
    }

    cr_enter_cs(my_id);
    mb = work(argc, argv);
    cr_request_fd(fd); // Blocked by the critical section
    rc = cr_status();
    if (rc != CR_STATE_PENDING) {
	printf("ERROR: cr_status() unexpectedly returned 0x%x\n", rc);
	exit(-1);
    }

    start_time = gettimeofday_us();
    cr_leave_cs(my_id);	// should cause checkpoint NOW
    fsync(fd);
    end_time = gettimeofday_us();

    rc = cr_status();
    if (rc != CR_STATE_IDLE) {
	printf("ERROR: cr_status() unexpectedly returned 0x%x\n", rc);
	exit(-1);
    }

    printf("Checkpoint w/ %d MB heap took %ld us\n", mb, (long)(end_time - start_time));
    (void)unlink(filename); // may fail silently

    return 0;
}

==> blcr-0.6.2.tar.gz: blcr-0.6.2/examples/io_bench/io_bench.c <==
/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2006, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: io_bench.c,v 1.3 2007/04/19 23:04:55 phargrov Exp $
 */

#define _LARGEFILE64_SOURCE 1	/* For O_LARGEFILE */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdint.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
 
#include "libcr.h"

extern uint64_t gettimeofday_us(void) {
  uint64_t retval;
  struct timeval tv;
  gettimeofday(&tv, NULL);
  retval = ((uint64_t)tv.tv_sec) * 1000000 + (uint64_t)tv.tv_usec;
  return retval;
}

int work(int argc, const char **argv) {
    size_t remain;
    int mb;

    if (argc != 2) {
      printf("ERROR: %s requires 1 argument, the memory size in megabytes\n", argv[0]);
      exit(1);
    }
    mb = atoi(argv[1]);
    if (mb <= 0) {
      printf("ERROR: '%s' is not a positive integer\n", argv[1]);
      exit(1);
    }

    remain = (size_t)mb * 1024 * 1024;
    while (remain) {
      size_t try;
      for (try = remain; try != 0; try /= 2) {
        char *ptr = malloc(try);
	if (ptr != NULL) {
	  size_t i;
	  /* Touch each page (more than once if page size > 4k */
	  for (i = 0; i < try; i += 4096) {
	    ptr[i] = (char)1;
	  }
	  remain -= try;
	  break;
	}
      }
      if (try == 0) {
        printf("ERROR: failed to allocate %d megabytes of memory\n", mb);
        exit(1);
      }
    }

    return mb;
}

int main(int argc, const char **argv)
{
    char filename[40];
    cr_client_id_t my_id;
    uint64_t start_time, end_time;
    int rc, fd, mb;

    setlinebuf(stdout);

    snprintf(filename, sizeof(filename), "context.%d", (int)getpid());
    fd = open(filename, O_WRONLY|O_CREAT|O_NOCTTY|O_NONBLOCK|O_TRUNC|O_LARGEFILE, 0400);
    if (fd < 0) {
	printf("ERROR: open(%s) failed: %s\n", filename, strerror(errno));
	exit(-1);
    }

    my_id = cr_init();
    if (my_id < 0) {
	printf("ERROR: cr_init() failed, returning %d\n", my_id);
	exit(-1);
    }

    cr_enter_cs(my_id);
    mb = work(argc, argv);
    cr_request_fd(fd); // Blocked by the critical section
    rc = cr_status();
    if (rc != CR_STATE_PENDING) {
	printf("ERROR: cr_status() unexpectedly returned 0x%x\n", rc);
	exit(-1);
    }

    start_time = gettimeofday_us();
    cr_leave_cs(my_id);	// should cause checkpoint NOW
    fsync(fd);
    end_time = gettimeofday_us();

    rc = cr_status();
    if (rc != CR_STATE_IDLE) {
	printf("ERROR: cr_status() unexpectedly returned 0x%x\n", rc);
	exit(-1);
    }

    printf("Checkpoint w/ %d MB heap took %ld us\n", mb, (long)(end_time - start_time));
    (void)unlink(filename); // may fail silently

    return 0;
}
